#!/usr/local/bin/perl -T

###############################################################################
#							FC_ADMIN.PL
#
# Version: 2.0.2
#
# Last Modified:  01 DEC 1998
#
# This program was written by Michael E. Schechter (mschechter@earthlink.net)
# in the course of heavily modifying Selena Sol's Groupware Calendar v3.0.
# Selena Sol can be reached at selena@extropia.com.  This particular file
# contains only the administration routines used by FrameCal.pl.
#
# You may copy this under the terms of the GNU General Public
# License or the Artistic License which is distributed with
# copies of Perl v5.x for UNIX.
#
# Please send working URLs to mschechter@earthlink.net.  This program is free,
# and will remain free.  Donations to defray the cost of the platform for
# distribution are greatly appreciated.  However, if you are student (of Perl
# or otherwise), or a non-profit organization, and feel you would like to
# make a donation, send it to your favorite charity instead.  If you would
# like to send me money, just drop me an email.
#
# Please see the revision history HTML document to learn all of the changes
# in the current version.  It can be found in the htdocs subdirectory of the
# distribution.  Please read the installation file also found therein.
###############################################################################

# First, tell Perl to bypass the buffer.

$| = 1;

# REQUIRE LIBRARIES #

# Now, let's require the necessary library and setup files using the
# CgiRequire subroutine found in this file.  The subroutine will provide
# us with an error message if any of the files can not be required properly.

# $lib is the path to the directory containing the library files that
# are used by this, and many other, scripts.  Note that the line below
# sets it using a relative path.  On some server setups, you will have to
# change this to an absolute path.

$lib = "../libs";
&CgiRequire("$lib/cgi-lib.pl");

# GET FORM DATA #

# Now we use the cgi-lib.pl subroutine ReadParse to get our form data
# into the hash %form_data.  Note that we rename it from its standard
# value of %in for clarification purposes.  After this, we will be able
# to get all incoming form data by referencing $form_data{'variablename'}.

&ReadParse(*form_data);

# DETERMINE CALENDAR #

# Since this script allows us to keep many different calendars using one
# script, we need to get the calendar to use from the form data.  This
# information will be stored in the key 'calendar'.  We read this information
# in and untaint it.  Note that only alphanumber and an optional / are allowed.
# In order to access this, the original link must have had ?calendar=some/dir
# in the URL.  If this value is omitted, we assume we are using calendar files
# stored in the script directory.

# After getting the calendar, we finish by requiring the calendar specific
# files.

if ($form_data{'calendar'} ne "") {
	$form_data{'calendar'} =~ /^(\w+\/?\w*)/o;
	$calendar_type = "../$1";
} else {
	$calendar_type = "./";
}
&CgiRequire("$calendar_type/calendar.setup", "$calendar_type/auth.setup");
&CgiRequire("$auth_lib/auth-lib.pl");

# IN MAINTENANCE #

# We have the option in the setup file to "shut down" the calendar by
# turning on the $in_maintenance flag.  This allows us to modify a small
# file and upload it, in effect stopping all processing while maintenance
# is performed.  If the value is true, we print out a message to the user
# and stop all further processing.

if ($in_maintenance) {
	&header ("Calendar Unavailable");
	print qq!
</form><center><h3>Calendar Unavailable</h3></center>
<blockquote>I'm sorry, but the calendar is currently unavailable due to
system maintenance.  Please try back in 15 minutes.</blockquote>!;
	&footer;
	exit;
}

# SESSION FILE #

# The session file is used on every pass through the script, to determine
# who is using it on the given pass.  So, we define a variable and set it
# to the value of session_file passed through the form data.  On the first
# pass, this will be blank, and will start us into the authorization process
# in a little while.

if ($form_data{'session_file'} ne "") {
	$form_data{'session_file'} =~ /^(\w+)/o;
	$session_file = $1;
}

# USER AUTHENTICATION #

# Okay, at this point, we want to find out if the user should actually be
# here.  We use the GetSessionInfo subroutine found in auth-lib.pl, which
# we have already required, to get the user to login, and authenticate them.
# We also chomp the session email, which removes a new line character if one
# exists on it.

($session_file, $session_username, $session_group, $session_name,
 $session_email) =
 &GetSessionInfo($session_file, "$admin_script?calendar=$calendar_name", *form_data);
chomp $session_email;

# DEFINE TEMP/LOCK FILES #

# Here we define the temp file and lock file that we will use while
# manipulating our user database files.

$temp_file = "$temp_lock_dir/user_file.temp";
$lock_file = "$temp_lock_dir/user_file.lock";

# WHAT DO YOU WANT ME TO DO? #

# Each button on the forms that are presented, and most links, provide a tag
# called 'action'.  This form data variable tells the script what button
# or type of link you selected, and thus, what you want to do.  So, we grab
# this data, and do it with a 'perl-esque' Case statement.  This is the
# end of the MAIN section of the script.  From here on out, we are just
# calling subroutines.

$action = $form_data{'action'};
CASE: {
	if ($action =~ /^User Admin/i)		{ do { user_admin_sub(1);	last CASE; } }
	if ($action =~ /^Set Access/i)		{ do { reg_sub(1);         	last CASE; } }
	if ($action =~ /^Delete User/i)		{ do { reg_sub(0);         	last CASE; } }
	if ($action =~ /^Register Users/i)  { do { user_admin_sub(0);	last CASE; } }
	if ($action =~ /^Register/i)		{ do { pend_sub(1);			last CASE; } }
	if ($action =~ /^Reject User/i)		{ do { pend_sub(0);			last CASE; } }
}

# SUB USER_ADMIN_SUB #

# If we have called this subroutine, the administrator has requested a list
# of registered or pending users to modify.  We pass a parameter, put into
# $choice, that lets us know whether we want to deal with registered users,
# or pending users.

sub user_admin_sub {
	my ($choice) = $_[0];
	my ($table,$name,$password,$username,$group,$email);

# We first check to make sure that the session_group is set to admin from
# our authentication routine, to be sure that an unauthorized person isn't
# trying to administer our users.  If the group is admin, we start printing
# out the HTML for administration routines.

	if ($session_group eq "admin") {
		&header ("User Administration");
		print "<center>\n";
		if ($choice) {
			print "<h3>Set User Access Levels Or Delete Users</h3>\n";
		} else {
			print "<h3>Register or Reject Pending Users</h3>\n";
		}

# Print out a help link with the appropriate help topic, if we are using online
# help.

		if ($use_help) {
			print "<p>";
			if ($choice) {
				&help_link ("7");
			} else {
				&help_link ("11");
			}
		}
		print "</center>";

# If our choice is to deal with registered users, we open up the auth_user_file,
# otherwise we open up the auth_alt_user_file, which holds our pending users.

		if ($choice) {
			open (AUTHFILE, "$auth_user_file") || &open_error($auth_user_file);
		} else {
			open (AUTHFILE, "$auth_alt_user_file") || &open_error($auth_alt_user_file);
		}

# We print out the beginning of our table, setting the headers of each column using
# our auth_names array, and providing a column to select users with.

		$table .= "<center><table border=\"2\" cellspacing=\"1\" cellpadding=\"2\">";
		$table .= "\n<tr>\n";
		$table .= "<th bgcolor=$apptcol valign=\"bottom\"><font color=$hitxt size=\"2\">Select</font></th>";
		foreach $name (@auth_names) {
			$table .= "<th bgcolor=$apptcol valign=\"bottom\"><font color=$hitxt size=\"2\">$name</font></th>\n";
		}
		$table .= "</tr>\n";
		print $table;

# We now go through our selected user file, one line at a time, and extract the
# information from it, providing one row in our table for each user.  We also
# provide a checkbox for each user, set to the username, for selection purposes.
# When we are done, we close the file.

		while (<AUTHFILE>) {
			chomp $_;
			($password, $username, $group, $name, $email) = split (/\|/,$_);
			$table = "";
			$table .= "<tr>\n";
			$table .= "<td align=\"center\">";
			$table .= "<input type=\"checkbox\" name=\"item_to_register\" ";
			$table .= "value=\"$username\"></td>\n";
			$table .= "<td><font size=2>$name</font></td>\n";
			$table .= "<td><font size=2>$username</font></td>\n";
			$table .= "<td><font size=2>$group</font></td>\n";
			$table .= "<td><font size=2><a href=\"mailto:$email\">$email</a></font></td>";
			$table .= "\n</tr>\n";
			print $table;
		}
		close (AUTHFILE);

# We now close our table, and provide radion buttons for setting the users to
# different user groups.  We set the default selected group to the default group
# in our auth.setup file.

		print "</table><p>";
		foreach $group ( ("public", "user", "admin") ) {
			if ($group ne "$auth_default_group") {
				print "<input type=\"radio\" name=\"group\" value=\"$group\">",
					  "<b>",ucfirst ("$group"),"</b>";
			} else {
				print "<input type=\"radio\" name=\"group\" value=\"$group\" checked>",
					  "<b>",ucfirst ("$group"),"</b>";
			}
		}
		print "<p>";

# Now we provide form buttons appropriate to the type of table we are using, one
# set for registered users, and another set for pending users.

		if ($choice) {
			print "<input type=\"submit\" name=\"action\" value=\"Set Access\">\n";
			print "<input type=\"submit\" name=\"action\" value=\"Delete User\">";
		} else {
			print "<input type=\"submit\" name=\"action\" value=\"Register\">\n";
			print "<input type=\"submit\" name=\"action\" value=\"Reject User\">";
		}
		print qq!
<input type="reset" value="Clear This Form">
</center></form>!;
	} else {

# If the group in the session file wasn't set to admin, we inform the user that
# they do not have the necessary access level to perform this action.  Note that
# there won't be buttons for non-authorized people to access this routine.  This
# is just a preventive measure for people trying to access this subroutine by
# typing in the query string manually.

		&header("Unauthorized Access");
		print qq!
<center><h3>Unauthorized Access</h3></center>\n<blockquote>
You do not have the correct access level to perform user administration.
Please hit your back button, and select a valid action from the screen.
</blockquote></form>!;
	}

# We are now done, so we print out our standard footer.

    &footer;
}

# SUB REG_SUB #

# If we have called this subroutine, it means the administrator has requested
# to delete a registered user, or change a registered user's access level.
# Which action is going to be performed is determined by the paramter
# that we pass it, which is stored in $reg_action.

sub reg_sub {
	my ($reg_action) = $_[0];
	my ($reg_users,$user_id,$email_address,$address);
	my ($subject,$message,@reg_del_users,@grepfields,@email_addresses);
	my (%reg_del_users,$i);

# First we check the session file information to be sure the requestor is
# indeed set to admin access.

	if ($session_group eq "admin") {

# If we are dealing with the administrator, we need to gather the form information.
# First we get the users the administrator wants to manipulate by splitting
# the item_to_register form item into the reg_del_users array.  We then use
# the items in this array as keys of a hash called reg_del_users for easier
# manipulation a little later.

		@reg_del_users = split ("\0", $form_data{'item_to_register'});
		foreach $i (@reg_del_users) {
			$reg_del_users{"$i"} = 1;
		}

# We concatenate the users in the reg_del_users array into a comma separated
# string for later output.

		$reg_users = join (", ", @reg_del_users);

# If the string doesn't contain anything, we didn't receive any users from
# the administrator, so we print out an error message to signify this, and
# then exit.

		if ($reg_users eq "") {
			&header ("No one to Update");
			print qq!
<center><h3>Update/Delete a User File Error</h3></center>
<blockquote>
I'm sorry, I was not able to modify the authorization file because
none of the check boxes on the table were selected, so I was
not sure which user to update or delete.  Would you please make sure
that you select a user.
Just hit your back button.  Thanks.</blockquote></form>!;
			&footer;
			exit;
		}

# If we made it to this point, we are ready to modify the users.  We get a lock
# file setup for our users file, open up a temp file, and open up the user
# file.  We then go though the file one line at a time.

		&GetFileLock ("$lock_file");
		open (TEMP, ">$temp_file") || &open_error("$temp_file");
		open (DATA, "$auth_user_file") || &open_error($auth_user_file);
		while (<DATA>) {

# On each pass we split the line into the grepfields array, and get
# the user's name into the user_id variable.

			@grepfields=split(/\|/,$_);
			$user_id = $grepfields[1];

# If the user is in our %reg_del_users hash, we grab and chomp the email address
# and push it onto an email addresses array.

			if ($reg_del_users{$user_id}) {
				$email_address = $grepfields[4];
				chomp $email_address;
				push (@email_addresses, $email_address);
				if ($reg_action) {

# If our action is to modify the users access level, we change the access level,
# and print the users information back into the temp file.  Otherwise, we do
# nothing.

					$grepfields[2] = "$form_data{'group'}";
					print TEMP join("\|", @grepfields);
				}
			} else {

# If the user we are looking at is not in our %reg_del_users hash, we print their
# information into the temp file.

					print TEMP "$_";
			}
		}

# When we get to the end of the file, we close our temp and user files up,
# remove the original user file, and replace it with the temp file.  We then
# release our file lock.

		close (TEMP);
		close (DATA);
		unlink ("$auth_user_file");
		rename ($temp_file, $auth_user_file);
		&ReleaseFileLock ("$lock_file");

# If we have chosen, in our auth.setup file, to notify people of changes in
# their registration status automatically, we require our mail-lib.pl file
# so we can send mail.  We then prepare our subject and message dependent
# on whether we have deleted users, or changed their access level.

		if ($notify_register && ($auth_admin_from_address ne "")) {
			&CgiRequire("$lib/mail-lib.pl");
			if ($reg_action) {
				$subject = "Calendar Access Level Confirmed";
				$message = "You have been successfully set to ";
				$message .= "$form_data{'group'} access with the ";
				$message .= "$cal_name calendar ($html_url).";
			} else {
				$subject = "Calendar Account Deleted";
				$message = "Your account has been deleted from the $cal_name calendar.";
			}

# After we have our email message together, we send it to each of the users in our
# email_addresses array.

			foreach $address (@email_addresses) {
				&send_mail($auth_admin_from_address, $address, $subject, $message);
			}
		}

# We now supply output to the administrator to let them know the action that
# we performed, and which users were affected.

		if (@reg_del_users > 1) {
			if ($reg_action) {
				$message = "The users $reg_users have been set to access level
							$form_data{'group'}.";
			} else {
				$message = "The users $reg_users have been deleted.";
			}
		} else {
			if ($reg_action) {
				$message = "The user $reg_users has been set to access level
							$form_data{'group'}.";
			} else {
				$message = "The user $reg_users has been deleted.";
			}
		}
		&header ("User Administration");
		print "<h3>\&nbsp\;\&nbsp\;$message</h3><p></form>";
	} else {

# If the person selecting this option is not at admin level access, we provide
# them with a message stating that they do not have the proper access to
# perform this action.

		&header("Unauthorized Access");
		print qq!
</form><center><h3>Unauthorized Access</h3></center>\n<blockquote>
You do not have the correct access level to perform user administration.
Please hit your back button, and select a valid action from the screen.
</blockquote>!;
	}

# We are now done, so we print out our standard footer.

	&footer;
}

# SUB PEND_SUB #

# If we have called this subroutine, it means the administrator has requested
# to delete a pending user, or register a pending user to full status.
# Which action is going to be performed is determined by the paramter
# that we pass it, which is stored in $pend_action.

sub pend_sub {
	my ($pend_action) = $_[0];
	my ($reg_users,$user_id,$email_address,$address);
	my ($subject,$message,@reg_del_users,@grepfields,@email_addresses);
	my (%reg_del_users,$i,@reg_users);

# First we check the session file information to be sure the requestor is
# indeed set to admin access.

	if ($session_group eq "admin") {

# If we are dealing with the administrator, we need to gather the form information.
# First we get the users the administrator wants to manipulate by splitting
# the item_to_register form item into the reg_del_users array.  We then use
# the items in this array as keys of a hash called reg_del_users for easier
# manipulation a little later.

		@reg_del_users = split ("\0", $form_data{'item_to_register'});
		foreach $i (@reg_del_users) {
			$reg_del_users{"$i"} = 1;
		}

# We concatenate the users in the reg_del_users array into a comma separated
# string for later output.

		$reg_users = join (", ", @reg_del_users);

# If the string doesn't contain anything, we didn't receive any users from
# the administrator, so we print out an error message to signify this, and
# then exit.

		if ($reg_users eq "") {
			&header ("No one to Register");
			print qq!
<center><h3>Register/Delete a User File Error</h3></center>
<blockquote>
I'm sorry, I was not able to modify the alternate authorization file because
none of the check boxes on the table were selected, so I was
not sure which user to register or delete.  Would you please make sure
that you select a user.
Just hit your back button.  Thanks.</blockquote></form>!;
			&footer;
			exit;
		}

# If we made it to this point, we are ready to modify the users.  We get a lock
# file setup for our alt_users file, open up a temp file, and open up the alt_user
# file.  We then go though the file one line at a time.

		&GetFileLock ("$lock_file");
		open (TEMP, ">$temp_file") || &open_error("$temp_file");
		open (DATA, "$auth_alt_user_file") || &open_error($auth_alt_user_file);
		while (<DATA>) {

# On each pass we split the line into the grepfields array, and get
# the user's name into the user_id variable.

			@grepfields=split(/\|/,$_);
			$user_id = $grepfields[1];

# If the user is in our %reg_del_users hash, we grab and chomp the email address
# and push it onto an email addresses array.

			if ($reg_del_users{$user_id}) {
				$email_address = $grepfields[4];
				chomp $email_address;
				push (@email_addresses, $email_address);

# If our action is to register the user, we set their group to the requested
# level, then push the "|" separated user information onto our @reg_users
# array for later manipulation.  If the action is to reject the user, we don't
# do anything.

				if ($pend_action) {
					$grepfields[2] = "$form_data{'group'}";
					push (@reg_users, join("\|", @grepfields));
				}
			} else {

# If the user we are looking at is not in our %reg_del_users array, that
# means we have not chosen to do anything with them at the current time,
# so we print their information into the temp file.

					print TEMP "$_";
			}
		}

# When we get to the end of the file, we close our temp and alt_user files up,
# remove the original alt_user file, and replace it with the temp file.

		close (TEMP);
		close (DATA);
		unlink ("$auth_alt_user_file");
		rename ($temp_file, $auth_alt_user_file);

# If our action is to register the users, we need to move them over into
# the regular user file.  We therefore open up the user file in append mode,
# and print each of our users that we selected to register into that file.
# We then close the file.

		if ($pend_action) {
			open (USERFILE, ">>$auth_user_file") || &open_error($auth_user_file);
			foreach $i (@reg_users) {
				print USERFILE $i;
			}
			close (USERFILE);
		}

# We are now done with our file manipulation, so we release our file lock.

		&ReleaseFileLock ("$lock_file");

# If we have chosen to notify users of the registration status via email,
# we require our mail-lib.pl file so we can use sendmail, then prepare
# the message and subject for the email message, dependent upon whether we
# are registering or rejecting the users.

		if ($notify_register && ($auth_admin_from_address ne "")) {
			&CgiRequire("$lib/mail-lib.pl");
			if ($pend_action) {
				$subject = "Calendar Registration Confirmed";
				$message = "Your account has been activated for the ";
				$message .= "$cal_name calendar ($html_url) with ";
				$message .= "$form_data{'group'} access.";
			} else {
				$subject = "Calendar Registration Rejected";
				$message = "Your account request for the $cal_name calendar ";
				$message .= "has been rejected.";
			}

# We then send the email message out to each of the users that are affected.

			foreach $address (@email_addresses) {
				&send_mail($auth_admin_from_address, $address, $subject, $message);
			}
		}

# We now prepare an email message for the administrator to specify what actions
# we have taken, and which users were affected.

		if (@reg_del_users > 1) {
			if ($pend_action) {
				$message = "Accounts have been activated for the users $reg_users ";
				$message .= "with an access level of $form_data{'group'}.";
			} else {
				$message = "The users $reg_users have been deleted.";
			}
		} else {
			if ($pend_action) {
				$message = "An account has been activated for the user $reg_users ";
				$message .= "with an access level of $form_data{'group'}.";
			} else {
				$message = "The user $reg_users has been deleted.";
			}
		}
		&header ("User Registration");
		print "<h3>\&nbsp\;\&nbsp\;$message</h3><p></form>";
	} else {

# If the person selecting this option is not at admin level access, we provide
# them with a message stating that they do not have the proper access to
# perform this action.

		&header("Unauthorized Access");
		print qq!
</form><center><h3>Unauthorized Access</h3></center>\n<blockquote>
You do not have the correct access level to perform user administration.
Please hit your back button, and select a valid action from the screen.
</blockquote>!;
	}

# We are now done, so we print out our standard footer.

	&footer;
}

# SUB CGIREQUIRE #

# This subroutine takes a list of files with path information that are needed
# by this script, and attempts to require them.

sub CgiRequire {
	my (@require_files) = @_;
	my ($file);

# Check to see if each file exists and is readable.
# If it is, require it.  If it is not, provide an error
# message for the file we could not require, and exit the program.

	foreach $file (@require_files) {
		if (-e "$file" && -r "$file") {
			require "$file";
		} else {
			&header("Require File Error");
			print qq!
I'm sorry, I was not able to open $file.  Would you
please check to make sure that you gave me a valid filename
and that the permissions on $require_file are set to allow me
access?</form></body></html>!;
			exit;
		}
	}
}

# SUB HEADER #

# This subroutine takes a title as an argument.  It then generates the
# content type information so the browser knows what it is receiving, and
# then opens up the HTML page.  It provides a javascript function that will
# be used by the help files and the search routine to display information.
# It also starts the standard form that will be used on most every page.

sub header {
	my ($title) = @_;
	if ($title eq "") {
		$title = "$cal_name Calendar";
	}
	print "Content-type: text/html\n\n";
	print "<html><head><title>$title</title>";
	if ($use_help) {
		print qq!
<script language="JavaScript" type="text/javascript">
<\!--
    function openNew(url) {
        var win = window.open(url, "fc_info", "scrollbars,height=225,width=400");
        win.focus();
    }
//-->
</script>\n!;
	}
	print qq!
</head>
<body$bodytag>
$basefont_tag
<form method="post" action="$admin_script">
<input type="hidden" name="session_file" value="$session_file">
<input type="hidden" name="calendar" value="$form_data{'calendar'}">!;
}

# SUB FOOTER #

# This subroutine is used to generate a standard footer for each HTML
# page, and properly close the HTML.

sub footer {
	print qq!
<p><font size="1">
For comments or questions, contact
<a href="mailto:$auth_admin_email_address?subject=Help">
Calendar Administrator</a></font></body></html>!;
}

# SUB HELP_LINK #

# This subroutine provides a nicely formatted help link, tying it into the javascript
# function we have defined in our header, given a help topic that will be accessed
# by the link.

sub help_link {
	my ($help_id) = @_;
	print qq!
<a href="javascript:openNew('$help_script?config_file=$help_setup&help_topic=$help_id')">
<b>H&nbsp;E&nbsp;L&nbsp;P</b></a>!;
}

# SUB GETFILELOCK #

# This subroutine gets a lock file so that a file can only be modified
# by one user at a time.  This is to prevent data mixing.  If you want
# to use flock, uncomment that line.  We only wait for 60 seconds if
# there is a lock file already in place, at which point we take over,
# assuming the old lock file is from a previous hang.

sub GetFileLock {
	my ($lock_file) = @_;
	my ($endtime) = 60;
	$endtime = time + $endtime;
	while (-e $lock_file && time < $endtime) {}
	open(LOCK_FILE, ">$lock_file");
#	flock(LOCK_FILE, 2);
}

# SUB RELEASEFILELOCK #

# This subroutine removes the passed lock file to free up a file
# for further editing.  If you want to use flock, uncomment that line.

sub ReleaseFileLock {
	my ($lock_file) = @_;
#	flock(LOCK_FILE, 8);
	close(LOCK_FILE);
	unlink($lock_file);
}

# SUB OPER_ERROR #

# If we have called this subroutine, it means that we have tried
# to open some file in the script, and failed.  We thus don't want
# to proceed, but we do want to provide the user with information
# on why we are not proceeding.

sub open_error {
	my ($filename) = @_;
	print qq!
</form><blockquote>I am sorry, but for some reason I was unable to open
<b>$filename</b>.  Would you please make sure that the filename is
correctly defined in the setup file, actually exists, and has the
right permissions relative to the web browser.  Thanks\!</blockquote>!;
	&footer;
	die;
}